package com.myntra.galera.test;

import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Properties;
import java.util.Random;
import java.util.logging.Level;
import java.util.logging.Logger;

public class SampleDBOperations {
	
	enum OperationResult{
		SUCCESS,
		FAILURE,
		INCOMPLETE,
		STALE
	}
	
	// JDBC driver name and database URL
	static final String JDBC_DRIVER = "com.mysql.jdbc.Driver";  
	static String DB_URL;
	//  Database credentials
	static String USER;
	static String PASSWORD;
	
	static final Random RANDOM = new Random(System.currentTimeMillis());
	static final Logger LOG = Logger.getAnonymousLogger();


	static {
		Properties prop = new Properties();
		InputStream input = null;

		try {

			input = SampleDBOperations.class.getClassLoader().getResourceAsStream("dbconfig.properties");
			prop.load(input);
			String host = prop.getProperty("dbhost", "localhost");
			String database = prop.getProperty("database", "galeratest");
			USER = prop.getProperty("dbuser","appuser");
			PASSWORD = prop.getProperty("password","password");
			DB_URL = "jdbc:mysql://" + host + "/" + database;

		} catch (IOException e) {
			LOG.log(Level.SEVERE, "excpetion while reading property file", e);
			System.out.println("couldn't read property file, exiting");
			System.exit(1);
		} finally {
			if (input != null) {
				try {
					input.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}

	public static Connection getConnection() throws SQLException{
		Connection conn = DriverManager.getConnection(DB_URL,USER,PASSWORD);
		conn.setAutoCommit(true);
		PreparedStatement stmt = conn.prepareStatement("set wsrep_causal_reads='ON'");
		stmt.executeQuery();
		return DriverManager.getConnection(DB_URL,USER,PASSWORD);
	}

	public int insertThreeColumns(int col1, int col2, int col3){
		int insertedId = -1;
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		try {
			conn = getConnection();

			stmt = conn.prepareStatement("insert into table1 (col1, col2, col3) value(?, ?, ?)");
			stmt.setInt(1, col1);
			stmt.setInt(2, col2);
			stmt.setInt(3, col3);
			stmt.executeUpdate();
			rs = stmt.getGeneratedKeys();
			if (rs.next()){
				insertedId=rs.getInt(1);
			}
		} catch (SQLException e) {
			LOG.log(Level.SEVERE, "exception while inserting", e);
		}finally{
			try{
				if(rs != null){
					rs.close();
				}
				if(stmt !=null){
					stmt.close();
				}
				if(conn !=null){
					conn.close();
				}
			}catch(SQLException e){
				LOG.log(Level.SEVERE, "excecption while inserting", e);
			}
		}
		return insertedId;
	}

	public int updateColumn3(int id, int col3){
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		int rowUpdatedCnt = -1;
		try {
			conn = getConnection();

			stmt = conn.prepareStatement("update table1 set col3 = ? where id = ?");
			stmt.setInt(1, col3);
			stmt.setInt(2, id);
			rowUpdatedCnt = stmt.executeUpdate();
		} catch (SQLException e) {
			LOG.log(Level.SEVERE, "exception while updating", e);
		}finally{
			try{
				if(rs != null){
					rs.close();
				}
				if(stmt !=null){
					stmt.close();
				}
				if(conn !=null){
					conn.close();
				}
			}catch(SQLException e){
				LOG.log(Level.SEVERE, "exception while updating", e);
			}
		}
		return rowUpdatedCnt;
	}

	public int[] selectThreeColumns(int id){
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		int[] cols = new int[3];
		try {
			conn = getConnection();

			stmt = conn.prepareStatement("select col1, col2, col3 from table1 where id = ?");
			stmt.setInt(1, id);
			rs = stmt.executeQuery();
			while(rs.next()){
				if(rs.isFirst()){
					cols[0] = rs.getInt("col1");
					cols[1] = rs.getInt("col2");
					cols[2] = rs.getInt("col3");
				}
			}
		} catch (SQLException e) {
			LOG.log(Level.SEVERE, "exception while selecting", e);
		}finally{
			try{
				if(rs != null){
					rs.close();
				}
				if(stmt !=null){
					stmt.close();
				}
				if(conn !=null){
					conn.close();
				}
			}catch(SQLException e){
				LOG.log(Level.SEVERE, "exception while selecting", e);
			}
		}
		return cols;
	}
	
	public OperationResult insertUpdateSelect(){
		int rand1 = RANDOM.nextInt(Integer.MAX_VALUE);
		int rand2 = RANDOM.nextInt(Integer.MAX_VALUE);
		int rand3 = RANDOM.nextInt(Integer.MAX_VALUE);
		int rand4 = RANDOM.nextInt(Integer.MAX_VALUE);
		int id = insertThreeColumns(rand1, rand2, rand3);
		if(id != -1){
			int rowUpdateCount = updateColumn3(id, rand4);
			if(rowUpdateCount != -1){
				int results[] = selectThreeColumns(id);
				if(results.length == 3){
					if(results[0] == rand1 && results[1] == rand2 && results[2] == rand4)
						return OperationResult.SUCCESS;
					else{
						if(results[2] == rand3){
							return OperationResult.STALE;
						}
						return OperationResult.FAILURE;
					}
				}
			}
		}
		return OperationResult.INCOMPLETE;
	}
	
	public void runTestInLoop(int loops){
		int successCnt = 0;
		int failureCnt = 0;
		int staleCnt = 0;
		int incompleteCnt = 0;
		for(int i = 0; i < loops; i++){
			OperationResult operationResult = insertUpdateSelect();
			switch (operationResult) {
			case SUCCESS:
				successCnt++;
				break;
			case FAILURE:
				failureCnt++;
				break;
			case STALE:
				staleCnt++;
				break;
			case INCOMPLETE:
				incompleteCnt++;
				break;
			default:
				break;
			}
		}
		System.out.println("No. of loops is " + loops + ", success count is " + successCnt + ", failure count is " + failureCnt + ", stale count is " + staleCnt + ", incomplete count is " + incompleteCnt);
	}

	public static void main(String[] args) {
		SampleDBOperations sampleDBOperations = new SampleDBOperations();
		sampleDBOperations.runTestInLoop(20);
	}
}